@@ -23,6 +23,7 @@ PROV = Namespace("http://www.w3.org/ns/prov#")
 MODELLDCATNO = Namespace("https://data.norge.no/vocabulary/modelldcatno#")
 FOAF = Namespace("http://xmlns.com/foaf/0.1/")
 SKOS = Namespace("http://www.w3.org/2004/02/skos/core#")
+XKOS = Namespace("http://rdf-vocabulary.ddialliance.org/xkos#")
 
 
 class InformationModel(Resource):
@@ -196,7 +197,13 @@ class InformationModel(Resource):
                 for _s, p, o in subject._to_graph().triples((None, None, None)):
                     self._g.add((_subject, p, o))
 
-                self._g.add((URIRef(self.identifier), DCT.subject, _subject,))
+                self._g.add(
+                    (
+                        URIRef(self.identifier),
+                        DCT.subject,
+                        _subject,
+                    )
+                )
 
     def _modelelements_to_graph(self: InformationModel) -> None:
 
@@ -210,7 +217,11 @@ class InformationModel(Resource):
                     _modelelement = BNode()
 
                 for _s, p, o in modelelement._to_graph().triples((None, None, None)):
-                    self._g.add((_modelelement, p, o))
+                    self._g.add(
+                        (_modelelement, p, o)
+                        if isinstance(_modelelement, BNode)
+                        else (_s, p, o)
+                    )
 
                 self._g.add(
                     (
@@ -233,7 +244,11 @@ class InformationModel(Resource):
             for _s, p, o in self.licensedocument._to_graph().triples(
                 (None, None, None)
             ):
-                self._g.add((_licensedocument, p, o))
+                self._g.add(
+                    (_licensedocument, p, o)
+                    if isinstance(_licensedocument, BNode)
+                    else (_s, p, o)
+                )
 
             self._g.add((URIRef(self.identifier), DCT.license, _licensedocument))
 
@@ -357,7 +372,13 @@ class ModelElement:
 
         if getattr(self, "title", None):
             for key in self.title:
-                self._g.add((_self, DCT.title, Literal(self.title[key], lang=key),))
+                self._g.add(
+                    (
+                        _self,
+                        DCT.title,
+                        Literal(self.title[key], lang=key),
+                    )
+                )
 
         if getattr(self, "dct_identifier", None):
             self._g.add((_self, DCT.identifier, Literal(self._dct_identifier)))
@@ -386,9 +407,19 @@ class ModelElement:
                     _has_property = BNode()
 
                 for _s, p, o in has_property._to_graph().triples((None, None, None)):
-                    self._g.add((_has_property, p, o))
+                    self._g.add(
+                        (_has_property, p, o)
+                        if isinstance(_has_property, BNode)
+                        else (_s, p, o)
+                    )
 
-                self._g.add((_self, MODELLDCATNO.hasProperty, _has_property,))
+                self._g.add(
+                    (
+                        _self,
+                        MODELLDCATNO.hasProperty,
+                        _has_property,
+                    )
+                )
 
 
 class ModelProperty:
@@ -530,7 +561,13 @@ class ModelProperty:
 
         if getattr(self, "title", None):
             for key in self.title:
-                self._g.add((_self, DCT.title, Literal(self.title[key], lang=key),))
+                self._g.add(
+                    (
+                        _self,
+                        DCT.title,
+                        Literal(self.title[key], lang=key),
+                    )
+                )
 
         if getattr(self, "subject", None):
 
@@ -554,9 +591,19 @@ class ModelProperty:
                     _has_type = BNode()
 
                 for _s, p, o in has_type._to_graph().triples((None, None, None)):
-                    self._g.add((_has_type, p, o))
+                    self._g.add(
+                        (_has_type, p, o)
+                        if isinstance(_has_type, BNode)
+                        else (_s, p, o)
+                    )
 
-                self._g.add((_self, MODELLDCATNO.hasType, _has_type,))
+                self._g.add(
+                    (
+                        _self,
+                        MODELLDCATNO.hasType,
+                        _has_type,
+                    )
+                )
 
 
 class Role(ModelProperty):
@@ -634,7 +681,11 @@ class Role(ModelProperty):
             for _s, p, o in self._has_object_type._to_graph().triples(
                 (None, None, None)
             ):
-                self._g.add((_has_object_type, p, o))
+                self._g.add(
+                    (_has_object_type, p, o)
+                    if isinstance(_has_object_type, BNode)
+                    else (_s, p, o)
+                )
 
             self._g.add((_self, MODELLDCATNO.hasObjectType, _has_object_type))
 
@@ -869,7 +920,9 @@ class SimpleType(ModelElement):
             self._g.add((_self, XSD.minInclusive, Literal(self.min_inclusive)))
 
         if getattr(self, "type_definition_reference", None):
-            self._g.add((_self, XSD.anyURI, URIRef(self.type_definition_reference)),)
+            self._g.add(
+                (_self, XSD.anyURI, URIRef(self.type_definition_reference)),
+            )
 
         if getattr(self, "pattern", None):
             self._g.add((_self, XSD.pattern, Literal(self.pattern)))
@@ -941,7 +994,9 @@ class Composition(ModelProperty):
                 _contains = BNode()
 
             for _s, p, o in self._contains._to_graph().triples((None, None, None)):
-                self._g.add((_contains, p, o))
+                self._g.add(
+                    (_contains, p, o) if isinstance(_contains, BNode) else (_s, p, o)
+                )
 
             self._g.add((_self, MODELLDCATNO.contains, _contains))
 
@@ -1012,7 +1067,11 @@ class Collection(ModelProperty):
                 _has_member = BNode()
 
             for _s, p, o in self._has_member._to_graph().triples((None, None, None)):
-                self._g.add((_has_member, p, o))
+                self._g.add(
+                    (_has_member, p, o)
+                    if isinstance(_has_member, BNode)
+                    else (_s, p, o)
+                )
 
             self._g.add((_self, MODELLDCATNO.hasMember, _has_member))
 
@@ -1083,7 +1142,9 @@ class Association(ModelProperty):
                 _refers_to = BNode()
 
             for _s, p, o in self._refers_to._to_graph().triples((None, None, None)):
-                self._g.add((_refers_to, p, o))
+                self._g.add(
+                    (_refers_to, p, o) if isinstance(_refers_to, BNode) else (_s, p, o)
+                )
 
             self._g.add((_self, MODELLDCATNO.refersTo, _refers_to))
 
@@ -1163,7 +1224,11 @@ class Choice(ModelProperty):
                     _has_some = BNode()
 
                 for _s, p, o in has_some._to_graph().triples((None, None, None)):
-                    self._g.add((_has_some, p, o))
+                    self._g.add(
+                        (_has_some, p, o)
+                        if isinstance(_has_some, BNode)
+                        else (_s, p, o)
+                    )
 
                 self._g.add((_self, MODELLDCATNO.hasSome, _has_some))
 
@@ -1171,12 +1236,20 @@ class Choice(ModelProperty):
 class Attribute(ModelProperty):
     """A class representing a modelldcatno:Attribute."""
 
-    __slots__ = ("_identifier", "_contains_object_type", "_has_simple_type")
+    __slots__ = (
+        "_identifier",
+        "_contains_object_type",
+        "_has_simple_type",
+        "_has_data_type",
+        "_has_value_from",
+    )
 
     _identifier: URI
     _contains_object_type: ObjectType
     _g: Graph
     _has_simple_type: SimpleType
+    _has_data_type: DataType
+    _has_value_from: CodeList
 
     def __init__(self) -> None:
         """Inits an object with default values."""
@@ -1202,6 +1275,26 @@ class Attribute(ModelProperty):
         """Set for has_simple_type."""
         self._has_simple_type = has_simple_type
 
+    @property
+    def has_data_type(self: Attribute) -> DataType:
+        """Get for has_data_type."""
+        return self._has_data_type
+
+    @has_data_type.setter
+    def has_data_type(self: Attribute, has_data_type: DataType) -> None:
+        """Set for has_data_type."""
+        self._has_data_type = has_data_type
+
+    @property
+    def has_value_from(self: Attribute) -> CodeList:
+        """Get for has_value_from."""
+        return self._has_value_from
+
+    @has_value_from.setter
+    def has_value_from(self: Attribute, has_value_from: CodeList) -> None:
+        """Set for has_value_from."""
+        self._has_value_from = has_value_from
+
     def to_rdf(
         self: Attribute, format: str = "turtle", encoding: Optional[str] = "utf-8"
     ) -> str:
@@ -1237,6 +1330,8 @@ class Attribute(ModelProperty):
 
         self._contains_object_type_to_graph(_self)
         self._has_simple_type_to_graph(_self)
+        self._has_data_type_to_graph(_self)
+        self._has_value_from_to_graph(_self)
 
         return self._g
 
@@ -1252,7 +1347,11 @@ class Attribute(ModelProperty):
             for _s, p, o in self._contains_object_type._to_graph().triples(
                 (None, None, None)
             ):
-                self._g.add((_contains_object_type, p, o))
+                self._g.add(
+                    (_contains_object_type, p, o)
+                    if isinstance(_contains_object_type, BNode)
+                    else (_s, p, o)
+                )
 
             self._g.add((_self, MODELLDCATNO.containsObjectType, _contains_object_type))
 
@@ -1268,10 +1367,53 @@ class Attribute(ModelProperty):
             for _s, p, o in self._has_simple_type._to_graph().triples(
                 (None, None, None)
             ):
-                self._g.add((_has_simple_type, p, o))
+                self._g.add(
+                    (_has_simple_type, p, o)
+                    if isinstance(_has_simple_type, BNode)
+                    else (_s, p, o)
+                )
 
             self._g.add((_self, MODELLDCATNO.hasSimpleType, _has_simple_type))
 
+    def _has_data_type_to_graph(self, _self: Any) -> None:
+
+        if getattr(self, "has_data_type", None):
+
+            if getattr(self._has_data_type, "identifier", None):
+                _has_data_type = URIRef(self._has_data_type.identifier)
+            else:
+                _has_data_type = BNode()
+
+            for _s, p, o in self._has_data_type._to_graph().triples((None, None, None)):
+                self._g.add(
+                    (_has_data_type, p, o)
+                    if isinstance(_has_data_type, BNode)
+                    else (_s, p, o)
+                )
+
+            self._g.add((_self, MODELLDCATNO.hasDataType, _has_data_type))
+
+    def _has_value_from_to_graph(self, _self: Any) -> None:
+
+        if getattr(self, "has_value_from", None):
+
+            _has_value_from = (
+                URIRef(self._has_value_from.identifier)
+                if getattr(self._has_value_from, "identifier", None)
+                else BNode()
+            )
+
+            for _s, p, o in self._has_value_from._to_graph().triples(
+                (None, None, None)
+            ):
+                self._g.add(
+                    (_has_value_from, p, o)
+                    if isinstance(_has_value_from, BNode)
+                    else (_s, p, o)
+                )
+
+            self._g.add((_self, MODELLDCATNO.hasValueFrom, _has_value_from))
+
 
 class Specialization(ModelProperty):
     """A class representing a modelldcatno:Specialization."""
@@ -1338,16 +1480,20 @@ class Specialization(ModelProperty):
         if getattr(self, "has_general_concept", None):
 
             if getattr(self._has_general_concept, "identifier", None):
-                has_general_concept = URIRef(self.has_general_concept.identifier)
+                _has_general_concept = URIRef(self.has_general_concept.identifier)
             else:
-                has_general_concept = BNode()
+                _has_general_concept = BNode()
 
             for _s, p, o in self._has_general_concept._to_graph().triples(
                 (None, None, None)
             ):
-                self._g.add((has_general_concept, p, o))
+                self._g.add(
+                    (_has_general_concept, p, o)
+                    if isinstance(_has_general_concept, BNode)
+                    else (_s, p, o)
+                )
 
-            self._g.add((_self, MODELLDCATNO.hasGeneralConcept, has_general_concept))
+            self._g.add((_self, MODELLDCATNO.hasGeneralConcept, _has_general_concept))
 
 
 class Realization(ModelProperty):
@@ -1416,7 +1562,11 @@ class Realization(ModelProperty):
                 _has_supplier = BNode()
 
             for _s, p, o in self._has_supplier._to_graph().triples((None, None, None)):
-                self._g.add((_has_supplier, p, o))
+                self._g.add(
+                    (_has_supplier, p, o)
+                    if isinstance(_has_supplier, BNode)
+                    else (_s, p, o)
+                )
 
             self._g.add((_self, MODELLDCATNO.hasSupplier, _has_supplier))
 
@@ -1489,6 +1639,363 @@ class Abstraction(ModelProperty):
             for _s, p, o in self._is_abstraction_of._to_graph().triples(
                 (None, None, None)
             ):
-                self._g.add((_is_abstraction_of, p, o))
+                self._g.add(
+                    (_is_abstraction_of, p, o)
+                    if isinstance(_is_abstraction_of, BNode)
+                    else (_s, p, o)
+                )
 
             self._g.add((_self, MODELLDCATNO.isAbstractionOf, _is_abstraction_of))
+
+
+class DataType(ModelElement):
+    """A class representing a modelldcatno:DataType."""
+
+    _identifier: URI
+    _dct_identifier: str
+    _g: Graph
+
+    def __init__(self) -> None:
+        """Inits an object with default values."""
+        super().__init__()
+
+    def to_rdf(
+        self: DataType, format: str = "turtle", encoding: Optional[str] = "utf-8"
+    ) -> str:
+        """Maps the data type to rdf.
+
+        Args:
+            format: a valid format. Default: turtle
+            encoding: the encoding to serialize into
+
+        Returns:
+            a rdf serialization as a string according to format.
+        """
+        return self._to_graph().serialize(format=format, encoding=encoding)
+
+    def _to_graph(
+        self: DataType, type: str = MODELLDCATNO.DataType, selfobject: Any = None
+    ) -> Graph:
+        """Returns the data type as graph.
+
+        Args:
+            type: type for identifying class. Default: MODELLDCATNO.DataType
+            selfobject: a bnode or URI passed from a subclass Default: None
+
+        Returns:
+            the object type graph
+        """
+        if getattr(self, "identifier", None):
+            _self = URIRef(self.identifier)
+        else:
+            _self = BNode()
+
+        super(DataType, self)._to_graph(MODELLDCATNO.DataType, _self)
+
+        return self._g
+
+
+class RootObjectType(ModelElement):
+    """A class representing a modelldcatno:RootObjectType."""
+
+    _identifier: URI
+    _dct_identifier: str
+    _g: Graph
+
+    def __init__(self) -> None:
+        """Inits an object with default values."""
+        super().__init__()
+
+    def to_rdf(
+        self: RootObjectType, format: str = "turtle", encoding: Optional[str] = "utf-8"
+    ) -> str:
+        """Maps the data type to rdf.
+
+        Args:
+            format: a valid format. Default: turtle
+            encoding: the encoding to serialize into
+
+        Returns:
+            a rdf serialization as a string according to format.
+        """
+        return self._to_graph().serialize(format=format, encoding=encoding)
+
+    def _to_graph(
+        self: RootObjectType,
+        type: str = MODELLDCATNO.RootObjectType,
+        selfobject: Any = None,
+    ) -> Graph:
+        """Returns the root object type as graph.
+
+        Args:
+            type: type for identifying class. Default: MODELLDCATNO.RootObjectType
+            selfobject: a bnode or URI passed from a subclass Default: None
+
+        Returns:
+            the root object type graph
+        """
+        if getattr(self, "identifier", None):
+            _self = URIRef(self.identifier)
+        else:
+            _self = BNode()
+
+        super(RootObjectType, self)._to_graph(MODELLDCATNO.RootObjectType, _self)
+
+        return self._g
+
+
+class CodeList(ModelElement):
+    """A class representing a modelldcatno:CodeList."""
+
+    _identifier: URI
+    _dct_identifier: str
+    _g: Graph
+
+    def __init__(self) -> None:
+        """Inits an object with default values."""
+        super().__init__()
+
+    def to_rdf(
+        self: CodeList, format: str = "turtle", encoding: Optional[str] = "utf-8"
+    ) -> str:
+        """Maps the code list to rdf.
+
+        Args:
+            format: a valid format. Default: turtle
+            encoding: the encoding to serialize into
+
+        Returns:
+            a rdf serialization as a string according to format.
+        """
+        return self._to_graph().serialize(format=format, encoding=encoding)
+
+    def _to_graph(
+        self: CodeList,
+        type: str = MODELLDCATNO.CodeList,
+        selfobject: Any = None,
+    ) -> Graph:
+        """Returns the root object type as graph.
+
+        Args:
+            type: type for identifying class. Default: MODELLDCATNO.CodeList
+            selfobject: a bnode or URI passed from a subclass Default: None
+
+        Returns:
+            the root object type graph
+        """
+        _self = (
+            URIRef(self.identifier) if getattr(self, "identifier", None) else BNode()
+        )
+
+        super(CodeList, self)._to_graph(MODELLDCATNO.CodeList, _self)
+
+        return self._g
+
+
+class CodeElement:
+    """A class representing a modelldcatno:CodeElement."""
+
+    __slots__ = (
+        "_identifier",
+        "_dct_identifier",
+        "_g",
+        "_type",
+        "_subject",
+        "_preflabel",
+        "_notation",
+        "_in_scheme",
+        "_top_concept_of",
+    )
+
+    _identifier: URI
+    _dct_identifier: str
+    _g: Graph
+    _type: str
+    _subject: Concept
+    _preflabel: dict
+    _notation: str
+    _in_scheme: List[CodeList]
+    _top_concept_of: List[CodeList]
+
+    def __init__(self) -> None:
+        """Inits an object with default values."""
+        self._type = MODELLDCATNO.CodeElement
+
+    @property
+    def identifier(self: CodeElement) -> str:
+        """Get for identifier."""
+        return self._identifier
+
+    @identifier.setter
+    def identifier(self: CodeElement, identifier: str) -> None:
+        """Set for identifier."""
+        self._identifier = URI(identifier)
+
+    @property
+    def dct_identifier(self: CodeElement) -> str:
+        """Get for dct_identifier."""
+        return self._dct_identifier
+
+    @dct_identifier.setter
+    def dct_identifier(self: CodeElement, dct_identifier: str) -> None:
+        """Set for dct_identifier."""
+        self._dct_identifier = dct_identifier
+
+    @property
+    def subject(self: CodeElement) -> Concept:
+        """Get for subject."""
+        return self._subject
+
+    @subject.setter
+    def subject(self: CodeElement, subject: Concept) -> None:
+        """Set for subject."""
+        self._subject = subject
+
+    @property
+    def preflabel(self: CodeElement) -> dict:
+        """Get for preflabel."""
+        return self._preflabel
+
+    @preflabel.setter
+    def preflabel(self: CodeElement, preflabel: dict) -> None:
+        """Set for preflabel."""
+        self._preflabel = preflabel
+
+    @property
+    def notation(self: CodeElement) -> str:
+        """Get for notation."""
+        return self._notation
+
+    @notation.setter
+    def notation(self: CodeElement, notation: str) -> None:
+        """Set for notation."""
+        self._notation = notation
+
+    @property
+    def in_scheme(self: CodeElement) -> List[CodeList]:
+        """Get for in_scheme."""
+        return self._in_scheme
+
+    @in_scheme.setter
+    def in_scheme(self: CodeElement, in_scheme: List[CodeList]) -> None:
+        """Set for in_scheme."""
+        self._in_scheme = in_scheme
+
+    @property
+    def top_concept_of(self: CodeElement) -> List[CodeList]:
+        """Get for top_concept_of."""
+        return self._top_concept_of
+
+    @top_concept_of.setter
+    def top_concept_of(self: CodeElement, top_concept_of: List[CodeList]) -> None:
+        """Set for top_concept_of."""
+        self._top_concept_of = top_concept_of
+
+    def to_rdf(
+        self: CodeElement, format: str = "turtle", encoding: Optional[str] = "utf-8"
+    ) -> str:
+        """Maps the code element to rdf.
+
+        Args:
+            format: a valid format. Default: turtle
+            encoding: the encoding to serialize into
+
+        Returns:
+            a rdf serialization as a string according to format.
+        """
+        return self._to_graph().serialize(format=format, encoding=encoding)
+
+    def _to_graph(self: CodeElement) -> Graph:
+        """Returns the code element as graph.
+
+        Returns:
+            the code element graph
+        """
+        _self = (
+            URIRef(self.identifier) if getattr(self, "identifier", None) else BNode()
+        )
+
+        # Set up graph and namespaces:
+        self._g = Graph()
+        self._g.bind("modelldcatno", MODELLDCATNO)
+        self._g.bind("dct", DCT)
+        self._g.bind("skos", SKOS)
+
+        self._g.add((_self, RDF.type, self._type))
+
+        if getattr(self, "dct_identifier", None):
+            self._g.add((_self, DCT.identifier, Literal(self._dct_identifier)))
+
+        if getattr(self, "notation", None):
+            self._g.add((_self, SKOS.notation, Literal(self._notation)))
+
+        self._preflabel_to_graph(_self)
+        self._subject_to_graph(_self)
+        self._in_scheme_to_graph(_self)
+        self._top_concept_of_to_graph(_self)
+
+        return self._g
+
+    def _subject_to_graph(self, _self: Any) -> None:
+
+        if getattr(self, "subject", None):
+
+            _subject = URIRef(self.subject.identifier)
+
+            for _s, p, o in self.subject._to_graph().triples((None, None, None)):
+                self._g.add((_subject, p, o))
+
+            self._g.add((_self, DCT.subject, _subject))
+
+    def _preflabel_to_graph(self, _self: Any) -> None:
+
+        if getattr(self, "preflabel", None):
+
+            for key in self.preflabel:
+                self._g.add(
+                    (
+                        _self,
+                        SKOS.prefLabel,
+                        Literal(self.preflabel[key], lang=key),
+                    )
+                )
+
+    def _in_scheme_to_graph(self, _self: Any) -> None:
+
+        if getattr(self, "in_scheme", None):
+
+            for in_scheme in self._in_scheme:
+
+                if getattr(in_scheme, "identifier", None):
+                    _in_scheme = URIRef(in_scheme.identifier)
+                else:
+                    _in_scheme = BNode()
+
+                for _s, p, o in in_scheme._to_graph().triples((None, None, None)):
+                    self._g.add(
+                        (_in_scheme, p, o)
+                        if isinstance(_in_scheme, BNode)
+                        else (_s, p, o)
+                    )
+
+                self._g.add((_self, SKOS.inScheme, _in_scheme))
+
+    def _top_concept_of_to_graph(self, _self: Any) -> None:
+
+        if getattr(self, "top_concept_of", None):
+
+            for top_concept_of in self._top_concept_of:
+
+                if getattr(top_concept_of, "identifier", None):
+                    _top_concept_of = URIRef(top_concept_of.identifier)
+                else:
+                    _top_concept_of = BNode()
+
+                for _s, p, o in top_concept_of._to_graph().triples((None, None, None)):
+                    self._g.add(
+                        (_top_concept_of, p, o)
+                        if isinstance(_top_concept_of, BNode)
+                        else (_s, p, o)
+                    )
+
+                self._g.add((_self, SKOS.topConceptOf, _top_concept_of))
